#!/usr/bin/env python

# Copyright (C) 2007, ENPC - INRIA - EDF R&D
#   Author(s): Meryem Ahmed de Biasi
#
# This file is part of the air quality modeling system Polyphemus.
#
# Polyphemus is developed in the INRIA - ENPC joint project-team CLIME and in
# the ENPC - EDF R&D joint laboratory CEREA.
#
# Polyphemus is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# Polyphemus is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# For more information, visit the Polyphemus web site:
#      http://cerea.enpc.fr/polyphemus/

# This module defines a series of classes to create a graphical user
# interface for Polyphemus.

import matplotlib
matplotlib.rcParams["backend"] = "Agg"
matplotlib.rcParams["interactive"] = False 
matplotlib.rcParams["figure.subplot.left"] = 0.05
matplotlib.rcParams["figure.subplot.right"] = 0.95
matplotlib.rcParams["figure.subplot.bottom"] = 0
matplotlib.rcParams["figure.subplot.top"] = 1

from Qt import * 
from QtCore import *
from QtGui import *

import os
from matplotlib.toolkits.basemap import Basemap
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas

from dates import *

from atmopy import *
from numpy import *
from pylab import *


class ConfigItem:
	"""
	Defines items of a configuration file.
	Each item has three attributes: a name, a section and a value.  This
	way one can modify the value of an item and still access the item
	through its name.
	"""
	def __init__(self, name, section, value):
		self.name = name
		self.section = section
		self.value = value

class PolyphemusConfig:
	"""
	Stores all data necessary for a simulation. Most attributes of the
	class are instances of the class ConfigItem whose values are set to an
	empty string. Another attribute is a list of all other attributes
	which allows to access them easily and automatically. The main aim of
	this class is to enables the user to save the configuration in a file
	or to load a configuration file.
	"""
	
	def __init__(self):
		"""
		Constructor.  Sets all values to a null string.
		"""
		self.path = ConfigItem("Polyphemus_path", "[paths]", "")
		self.meteo = ConfigItem("Meteorological_file", "[paths]", "")
		self.ground = ConfigItem("Ground_data", "[paths]", "" )
		self.date1 = ConfigItem("From", "[domain]", "")
		self.date2 = ConfigItem("To", "[domain]", "")
		self.x_min = ConfigItem("x_min", "[domain]", "")
		self.Delta_x = ConfigItem("Delta_x", "[domain]", "")
		self.Nx = ConfigItem("Nx", "[domain]", "")
		self.y_min = ConfigItem("y_min", "[domain]", "")
		self.Delta_y = ConfigItem("Delta_y", "[domain]", "")
		self.Ny = ConfigItem("Ny", "[domain]", "")
		self.Nz = ConfigItem("Nz", "[domain]", "")
		self.levels = ConfigItem("Levels", "[domain]", "")
		self.species = ConfigItem("Species", "[species]", "")
		self.source = ConfigItem("Source_file", "[species]", "")
		self.initial = ConfigItem("Initial_quantity", "[species]","")
		self.parameter = ConfigItem("Kz_parameterization", "[Kz]", "")
		
		self.list = [self.path, self.meteo, self.ground, self.date1,
			     self.date2, self.x_min, self.Delta_x, self.Nx,
			     self.y_min, self.Delta_y, self.Ny, self.Nz,
			     self.levels, self.species, self.source,
			     self.initial, self.parameter]

	def load(self, filename):
		"""
		Loads a configuration file. The file is read and the value of
		each ConfigItem is modified in accordance to the values given
		in the file.

		@type filename: String.
		@param filename: The name of the file to load.
		"""
		import talos
		config_file = talos.ConfigStream(filename)
		for item in self.list:
			item.value = config_file.GetString(item.name, item.section)



class PolyphemusDisplay(FigureCanvas):
	"""
	Displays concentrations on a Basemap.
	"""
	
	def __init__(self, parent, width, height, dpi, config):
		"""
		Constructor.

		@type parent: QWidget.
		@param parent: The parent widget of PolyphemusDisplay.

		@type width: integer.
		@param width: The width of the figure in inches. 
		
		@type height: integer.
		@param height: The height of the figure in inches. 

		@type dpi: integer.
		@param dpi: The number of dots per inches in the
		figure.

		@type config: String.
		@param config: The configuration file used to display the
		figure.
		"""
		self.width = width
		self.height = height
		self.dpi = dpi
		self.fig = figure(figsize=(self.width, self.height),
				  dpi = self.dpi, facecolor = 'w',
					edgecolor = 'w')
		self.axes = self.fig.add_axes([0, 0, 1, 1])
		FigureCanvas.__init__(self, self.fig)
		self.setParent(parent)
		xticks([])
		yticks([])
		self.configuration = config   # configuration file.
		self.step = 0                 # time step displayed.
		self.level = 0                # level displayed.
		self.map = None
		self.data = None
		FigureCanvas.setSizePolicy(self,
					   QSizePolicy.Expanding,
					   QSizePolicy.Expanding)
		FigureCanvas.updateGeometry(self)
		self.col = None

	def dispcf(self):
		"""
		Displays a binary file on a map using talos functions.
		The binary file and domain description are given in the
		attribute 'config' of the class.
		"""	
		if self.map is None: 
			self.map = display.getm(self.config, cbar = False) 
		if self.data is None:
			self.data = display.getd(self.config)
		
		self.fig.clf()
		self.axes.cla()
		if self.col != None:
			del self.col
				
		xrange, yrange = self.map.makegrid(self.data.shape[3],
						   self.data.shape[2])		
		self.map.contourf(xrange, yrange,
				  self.data[self.step, self.level])
		self.map.drawcountries()
		self.map.drawcoastlines()

		self.col = colorbar(orientation = 'horizontal')
		self.draw()
		

	def clear(self):
		"""
		Clears the figure.
		"""
		self.fig.clf()
		self.axes.cla()
		self.draw()
		self.map = None
		self.data = None
		self.col = None
		
	

class LineEdit(QLineEdit):
	"""
	Stores a value.
	The friend of a LineEdit is a ConfigItem whose value is displayed in
	the line. There is a two way relationship between a LineEdit and its
	friend: when the value of one (the value of a LineEdit being the value
	displayed) is modified by the user, the value of the other is modified
	too so that they always match. Functions 'print_friend' and 'update'
	represent this relationship.
	"""
	
	def __init__(self, content, parent, friend):
		"""
		Constructor.

		@type content: string.
		@param content: The text entered in the line when it is
		created. 

		@type parent: QWidget.
		@param top: The parent widget.

		@type friend: ConfigItem.
		@param friend: The ConfigItem whose value managed in the line
		edit. 
		"""
		QLineEdit.__init__(self, content, parent)
		self.setAlignment(Qt.AlignLeft)
		self.friend = friend
		self.connect(self, SIGNAL("textChanged(const QString &)"),
			     self.update)
		
	def print_friend(self):
		"""
		Changes the value displayed on the line when the value of the
		friend is modified. 
		"""
		self.setText(str(self.friend.value))

	def update(self, s):
		"""
		Changes the value of the friend when the value on the line
		edit is modified.
		"""
		if s != "":
			self.friend.value = str(s)

	def clear(self):
		"""
		Clears the LineEdit.
		"""
		self.friend.value = ""
		QLineEdit.clear(self)
		

class FileSelector(QWidget):
	"""
	Allows the user to select a file.
	Contains a text, a LineEdit whose content is the file selected and a
	button which opens a QFileDialog. There is thus two ways to select a
	file: either by entering its path and name in the LineEdit or with the
	file dialog.
	"""

	def __init__(self, content, parent, friend, filt, caption):
		"""
		Constructor.

		@type content: string.
		@param content: The text entered in the line when it is
		created. 

		@type parent: QWidget.
		@param top: The parent widget.

		@type friend: ConfigItem.
		@param friend: The ConfigItem whose value managed in the line
		edit. 

		@type filter: String.
		@param filter: The filter used when selecting the file (for
		example 'Data files (*.dat)').

		@type caption: String.
		@param caption: The caption displayed when the dialog is
		opened. 
		"""
		QWidget.__init__(self, parent)
		self.friend = friend
		self.filter = filt
		self.caption = caption
		self.grid = QGridLayout(self)
		self.label = QLabel(content, self)
		self.grid.addWidget(self.label, 0, 0)
		self.line = LineEdit("", self, self.friend)
		self.grid.addWidget(self.line, 0, 1)
		icon = QIcon(QPixmap("images/dossier-ouvert.png"))
		self.button = QPushButton(icon, "", self)
		self.grid.addWidget(self.button, 0, 2)
		self.connect(self.button, SIGNAL("clicked()"), self.select)

	def select(self):
		"""
		Opens a file dialog and modifies the content off the line
		edit to match the file selected.
		"""
		f = self.line.text()
		if f != "":
			s = QFileDialog.getOpenFileName(self, self.caption, 
							f, self.filter)
		else:
			s = QFileDialog.getOpenFileName(self, self.caption, 
							"./", self.filter)
		try:
			self.line.setText(str(s))
		except:
			pass
		
	def print_friend(self):
		"""
		Changes the value displayed on the line edit when the value of the
		friend is modified. 
		"""
		self.line.print_friend()
		
	def clear(self):
		"""
		Clears the content of the line edit.
		"""
		self.line.clear()


class DirSelector(QWidget):
	"""
	Similar to class FileSelector except that the file dialog only
	allows to select a directory.
	"""
	
	def __init__(self, content, parent, friend, caption):
		"""
		Constructor.

		@type content: string.
		@param content: The text entered in the line when it is
		created. 

		@type parent: QWidget.
		@param parent: The parent widget.

		@type friend: ConfigItem.
		@param friend: The ConfigItem whose value is managed in the line
		edit. 

		@type caption: String.
		@param caption: The caption displayed when the dialog is
		opened. 
		"""
		QWidget.__init__(self, parent)
		self.friend = friend
		self.caption = caption
		self.grid = QGridLayout(self)
		self.label = QLabel(content, self)
		self.grid.addWidget(self.label, 0, 0)
		self.line = LineEdit("", self, self.friend)
		self.grid.addWidget(self.line, 0, 1)
		icon = QIcon(QPixmap("images/dossier-ouvert.png"))
		self.button = QPushButton(icon, "", self)
		self.grid.addWidget(self.button, 0, 2)
		self.connect(self.button, SIGNAL("clicked()"), self.select)

	def select(self):
		"""
		Opens a file dialog and modifies the content of the line
		edit to match the directory selected.
		"""
		f = self.friend.value
		if f != "":
			s = QFileDialog.getExistingDirectory(self,
							     self.caption, f)
		else:
			s = QFileDialog.getExistingDirectory(self, 
							     self.caption, ".")
		try:
			self.line.setText(str(s))
		except:
			pass	
	

	def print_friend(self):
		"""
		Changes the value displayed on the line edit when the value of the
		friend is modified. 
		"""
		self.line.print_friend()	

	def clear(self):
		"""
		Clears the content of the line edit.
		"""
		self.line.clear()


class Section(QWidget):
	"""
	Class used to define some parameters of the simulation.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the various
		line edits' friends.
		"""
		QWidget.__init__(self, parent)
		self.config = config
		self.lineList = []

	def print_friend(self):
		"""
		Changes the value displayed on the lines when the value of
		their friends is modified.  
		"""
		for l in self.lineList:
			l.print_friend()

	def clear(self):
		"""
		Clears the content of every line edit of the section.
		"""
		for line in self.lineList:
			line.clear()	


		
class DialogFile(QDialog):
	"""
	Class used to open a temporary dialog which allows to reload the
	source file.
	"""
	
	def __init__(self, top, parent, friend, filename, content):
		"""
		Constructor.

		@type top: QWidget.
		@param top: The top-level widget. It is used to modify the
		source file.

		@type parent: QWidget.
		@param parent: The parent widget.

		@type friend: LineEdit.
		@param friend: The LineEdit whose value can be modified.

		@type filename: String.
		@param: the file to load.

		@type content: String.
		@param content: The text to display in the dialog.
		"""
		QDialog.__init__(self, parent)
		self.setModal(True)
		self.top = top
		self.friend = friend
		self.file = filename
		self.grid = QGridLayout(self)
		self.label = QLabel(content, self)
		self.grid.addWidget(self.label, 0, 0, 1, 4)
		self.pushYes = QPushButton ("Yes", self)
		self.grid.addWidget(self.pushYes, 1, 0)
		self.pushNo = QPushButton ("No", self)
		self.grid.addWidget(self.pushNo, 1, 3)
		self.connect(self.pushYes, SIGNAL("clicked()"), self.load)
		self.connect(self.pushNo, SIGNAL("clicked()"), self.close)

	def load(self):
		"""
		Allows the user to select the file (in case it was saved with
		another name) and loads it.
		"""
		s = QFileDialog.getOpenFileName(self,
				"Select the source file to load.",
				self.friend.text(),
			        "Data file(*.dat);;All files (*)")
		try:
			self.friend.setText(str(s))	
		except:
			pass
		self.close()	


class HelpBox(QDialog):
	"""
	Class used to open a temporary dialog with information.
	"""
	
	def __init__(self, parent, textfile):
		"""
		Constructor.
		
		@type parent: QWidget.
		@param parent: The parent widget.

		@type textfile: String.
		@param textfile: The file to display in the help box.
		"""
		QDialog.__init__(self, parent)
		self.setModal(False)
		self.grid = QGridLayout(self)
		self.grid.setRowMinimumHeight(1, 20)
		self.text = QTextBrowser(self)
		self.text.setSearchPaths([".", "help/"])
		self.text.setSource(QUrl(textfile))
		self.text.show()
		self.grid.addWidget(self.text, 0, 0, 1, 3)
		self.push = QPushButton ("Close", self)
		self.grid.addWidget(self.push, 2, 1)
		self.connect(self.push, SIGNAL("clicked()"), self.close)
		self.show()


# Classes to create all elements of the window. This classes exist so that the
# modification of an element can be done outside the creation of the
# MainWindow.
		
class SelectPath(DirSelector):
	"""
	DirSelector to select the path to Polyphemus.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
		The values of content and captions are given.
		"""
		DirSelector.__init__(self, "Path to Polyphemus", parent,
		config.path, "Select the path to Polyphemus directory")
		self.lineList = [self.line]

	def resize(self, size):
		sp = size.width() / 6
		self.grid.setColumnMinimumWidth(0, 2 * sp)
		self.grid.setColumnMinimumWidth(1, 3 * sp)
		self.grid.setColumnMinimumWidth(2, sp)

		
class SectionDomain(Section):
	"""
	Section to define the domain (in space) of the simulation.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the various
		line edits' friends.
		"""
		Section.__init__(self, parent, config)
		self.grid = QGridLayout(self)

		width = self.width() / 6
		for i in range(6):
			self.grid.setColumnMinimumWidth(i, width)

		self.labelxmin = QLabel("x_min = ", self)
		self.grid.addWidget(self.labelxmin, 0, 0)
		self.lineEditxmin = LineEdit("", self, self.config.x_min)
		self.grid.addWidget(self.lineEditxmin, 0, 1)
		self.labelDeltax = QLabel("Delta_x = ", self)
		self.grid.addWidget(self.labelDeltax, 0, 2)
		self.lineEditDeltax = LineEdit("", self, self.config.Delta_x)
		self.grid.addWidget(self.lineEditDeltax, 0, 3)
		self.labelNx = QLabel("Nx = ", self)
		self.grid.addWidget(self.labelNx, 0, 4)
		self.lineEditNx = LineEdit("", self, self.config.Nx)
		self.grid.addWidget(self.lineEditNx, 0, 5)
		
		self.labelymin = QLabel("y_min = ", self)
		self.grid.addWidget(self.labelymin, 1, 0)
		self.lineEditymin = LineEdit("", self, self.config.y_min)
		self.grid.addWidget(self.lineEditymin, 1, 1)
		self.labelDeltay = QLabel("Delta_y = ", self)
		self.grid.addWidget(self.labelDeltay, 1, 2)
		self.lineEditDeltay = LineEdit("", self, self.config.Delta_y)
		self.grid.addWidget(self.lineEditDeltay, 1, 3)
		self.labelNy = QLabel("Ny = ", self)
		self.grid.addWidget(self.labelNy, 1, 4)
		self.lineEditNy = LineEdit("", self, self.config.Ny)
		self.grid.addWidget(self.lineEditNy, 1, 5)
		
		self.labelNz = QLabel("Nz = ", self)
		self.grid.addWidget(self.labelNz, 2, 0)
		self.lineEditNz = LineEdit("", self, self.config.Nz)
		self.grid.addWidget(self.lineEditNz, 2, 1)

		self.selectLevels = FileSelector("Levels :", self,
		self.config.levels, "Data file (*.dat);; All files (*)",
		"Select the levels file")
		self.grid.addWidget(self.selectLevels, 2, 2, 1, 4)

		self.lineList = [self.lineEditxmin, self.lineEditDeltax,
				  self.lineEditNx, self.lineEditymin,
				  self.lineEditDeltay, self.lineEditNy,
				  self.lineEditNz, self.selectLevels.line]


class SectionDate(Section):
	"""
	Section to define the dates.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friend of
		the line edit (config.date).
		"""
		Section.__init__(self, parent, config)
		self.grid = QGridLayout(self)
		self.label_from = QLabel("From: ", self)
		self.grid.addWidget(self.label_from, 0, 0)
		self.line_from = LineEdit("", self, self.config.date1)
		self.grid.addWidget(self.line_from, 0, 1)
		self.label_to = QLabel("To: ", self)
		self.grid.addWidget(self.label_to, 0, 3)
		self.line_to = LineEdit("", self, self.config.date2)
		self.grid.addWidget(self.line_to, 0, 4)
		self.lineList = [self.line_from, self.line_to]


class DataPath(QWidget):
	"""
	Widget to select the paths to raw data.
	"""	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friend of
		the line edit (config.data).
		"""
		QWidget.__init__(self, parent)
		self.grid = QGridLayout(self)
		self.meteo = FileSelector("MM5 file: ", self,
					  config.meteo, "All files (*)",
					  "Select the path to the MM5 file.")
		self.ground = DirSelector("Ground data", self,
					  config.ground,
					  "Select the path to USGS raw data")
		self.grid.addWidget(self.meteo, 0, 0)
		self.grid.addWidget(self.ground, 1, 0)
		self.lineList = [self.meteo.line, self.ground.line]

	def clear(self):
		"""
		Clears the data.
		"""
		for line in self.lineList:
			line.clear()


class SectionSpecies(Section):
	"""
	Section to define the species used, its sources and its initial
	condition. Note that only one species can be considered.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friends of
		the various line edits.
		"""
		Section.__init__(self, parent, config)
		self.grid = QGridLayout(self)

		width = self.width() / 5
 		for i in range(5):
 			self.grid.setColumnMinimumWidth(i, width)
		
		self.labelSpecies = QLabel("Species: ", self)
		self.grid.addWidget(self.labelSpecies, 0, 0)
		self.lineEditSpecies = LineEdit("", self, self.config.species)
		self.grid.addWidget(self.lineEditSpecies, 0, 1)
		self.labelInit = QLabel("Initial quantity: ", self)
		self.grid.addWidget(self.labelInit, 0, 2)
		self.lineEditInit = LineEdit("", self, self.config.initial)
		self.grid.addWidget(self.lineEditInit, 0, 3)
		self.selectSource = FileSelector("Source:", self,
		self.config.source, "Data file (*.dat);; All files (*)",
		"Select the source file")
		self.grid.addWidget(self.selectSource, 1, 0, 1, -1)
		self.lineList = [self.lineEditSpecies, self.selectSource.line,
				 self.lineEditInit]
	

class SectionKz(Section):
	"""
	Section to define the Kz parameterization.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friend of
		the line edit (config.param).
		"""
		Section.__init__(self, parent, config)
		self.grid = QGridLayout(self)
		self.friend = config.parameter
		self.label = QLabel("Kz parameterization: ", self)
		self.grid.addWidget(self.label, 0, 0)
		self.comboBoxKz = QComboBox(self)
		self.comboBoxKz.setEditable(False)
		self.comboBoxKz.insertItem(0, "Louis")
		self.comboBoxKz.insertItem(1, "Troen and Mahrt")
		self.grid.addWidget(self.comboBoxKz, 0, 1, 0, 2)
		self.lineList=[]
		self.connect(self.comboBoxKz,
			     SIGNAL("textChanged(const QString &)"),
			     self.update)

	def print_friend(self):
		"""
		Changes the value displayed on the combo box when the value of
		the friend is modified.
		"""
		if self.friend.value == "TM":
			self.comboBoxKz.setCurrentIndex(1)
		else:
			self.comboBoxKz.setCurrentIndex(0)

	def update(self, s):
		"""
		Changes the value of the friend when the value on the combo
		box is modified.
		"""
		if s == "Troen and Mahrt":
			self.friend.value = "TM"
		else:
			self.friend.value = "Louis"

	def clear(self):
		"""
		Reset the combo box to its default value ('Louis')
		"""
		self.comboBoxKz.setCurrentIndex(0)
		

class ImageViewer(QWidget):
	"""
	Holds a PolyphemusDisplay and lines and buttons to control the figure
	displayed.
	"""
	
	def __init__(self, parent, top, width, height, dpi, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type height: integer.
		@param height: The height of the figure in inches. Default
		value is 5.

		@type width: integer.
		@param width: The width of the figure in inches. Default
		value is 4.
		
		@type dpi: integer.
		@param dpi: The number of dots per inches in the
		figure. Default value is 100.

		@type config: PolyphemusConfig.
		@param config: The configuration file used to display images.
		"""
		QWidget.__init__(self, parent)
		# Creation of the layout and of its content.
		self.grid = QGridLayout(self)
		self.dpi = dpi
		width = width * dpi / 4
		for i in range(4):
			self.grid.setColumnMinimumWidth(i, width)
		self.grid.setRowMinimumHeight(1, 10)	
		self.top = top
		self.labelDate = QLabel("Date shown: ", self)
		self.grid.addWidget(self.labelDate, 0, 0)
		self.lineDate = QLineEdit("", self)
		self.lineDate.setAlignment(Qt.AlignLeft)
		self.lineDate.setReadOnly(True)
		self.grid.addWidget(self.lineDate, 0, 1, 1, 3)
		self.polyphemusDisplay = PolyphemusDisplay(self, width,
							   height, dpi, config)
		self.grid.addWidget(self.polyphemusDisplay, 1, 0, 1, 4)
		self.labelStep = QLabel("Visualization for time step: ", self)
		self.grid.addWidget(self.labelStep, 2, 1, 1, 2)
		self.lineEditStep = QLineEdit("0", self)
		self.grid.addWidget(self.lineEditStep, 2, 3)
		self.labelLevel = QLabel("Visualization for level: ", self)
		self.grid.addWidget(self.labelLevel, 3, 1, 1, 2)
		self.lineEditLevel = QLineEdit("0", self)
		self.grid.addWidget(self.lineEditLevel, 3, 3)
		self.pushBackward = QPushButton(" << ", self)
		self.grid.addWidget(self.pushBackward, 4, 0, 1, 2)
		self.pushForward = QPushButton(" >> ", self)
		self.grid.addWidget(self.pushForward, 4, 2, 1, 2)

		# Creation of the actions.
		self.back_action = QAction("Backward", self)
		self.back_action.setShortcut(Qt.CTRL + Qt.Key_PageDown)
		self.connect(self.back_action, SIGNAL("triggered()"),
			     self.backward)
		self.for_action = QAction("Forward", self)
		self.for_action.setShortcut(Qt.CTRL + Qt.Key_PageUp)
		self.connect(self.for_action, SIGNAL("triggered()"),
			     self.forward)

		# Connection of the signal and slots.
		self.connect(self.pushBackward, SIGNAL("clicked()"),
			     self.backward)
		self.connect(self.pushForward, SIGNAL("clicked()"),
			     self.forward)
		self.connect(self.lineEditLevel,
			     SIGNAL("textChanged(const QString &)"),
			     self.disp)
		self.connect(self.lineEditStep,
			     SIGNAL("textChanged(const QString &)"),
			     self.disp)

	def print_friend(self):
		"""
		Changes the value displayed on the lines when the value of
		their friends is modified. 
		"""
		self.lineEditStep.setText(str(self.polyphemusDisplay.step))
		self.lineEditLevel.setText(str(self.polyphemusDisplay.level))
		self.date = Date(str(self.top.date.line_from.text()))
		self.date.add_hours(self.polyphemusDisplay.step)
		self.lineDate.setText(self.date.formate())

	def disp(self, s):
		"""
		Display the figure when the value on the lines are modified.

		@type s: String.
		@param s: The string entered in one of the line. The function
		only checks that it is not a null string.
		"""
		if s != "":
			step_max = self.polyphemusDisplay.data.shape[0] - 1
			level_max = self.polyphemusDisplay.data.shape[1] - 1	
			st = self.lineEditStep.text().toInt()[0]
			if 0 <= st and st <= step_max:
				self.polyphemusDisplay.step = st
			elif st < 0:
				self.polyphemusDisplay.step = 0
			else:
				self.polyphemusDisplay.step = step_max
			l = self.lineEditLevel.text().toInt()[0]
			if 0 <= l and l <= level_max:
				self.polyphemusDisplay.level = l
			elif l < 0:
				self.polyphemusDisplay.level = 0
			else:
				self.polyphemusDisplay.level = level_max
			self.polyphemusDisplay.dispcf()
			self.print_friend()
		
	def forward(self):
		"""
		Increases the value of PolyphemusDisplay.step by one if
		possible and display the updated figure. If it is not
		possible, it does nothing.
		"""
		M = self.polyphemusDisplay.data.shape[0] - 1
		if self.polyphemusDisplay.step != M:
			self.polyphemusDisplay.step += 1
			self.polyphemusDisplay.dispcf()
			self.print_friend()

	def backward(self):
		"""
		Decreases the value of PolyphemusDisplay.step by one if
		possible and display the updated figure. If it is not
		possible, it does nothing.
		"""
		if self.polyphemusDisplay.step != 0:
			self.polyphemusDisplay.step -= 1
			self.polyphemusDisplay.dispcf()
			self.print_friend()
			
	def clear(self):
		"""
		Clears the ImageViewer.
		"""
		self.lineEditStep.setText("0")
		self.lineEditLevel.setText("0")
		self.polyphemusDisplay.clear()
		self.lineDate.setText("")

	def save(self):
		"""
		Opens a file dialog to save the image.
		"""
		try:
			f = str(QFileDialog(self.top.file).
				getSaveFileName(self.top,
			        "Select the file in which to save the image.",
				".", "Images (*.png);;Image (*.eps)" \
						+ ";;All files (*)"))
			if f != "":
				title(str(self.lineDate.text()))
				savefig(f, dpi = self.dpi)
		except:
			pass


class GroupButton(QWidget):
	"""
	Creates four buttons to launch the different parts of the simulation,
	clear the dialog and quit the dialog.
	"""
	
	def __init__(self, top, parent):
		"""
		Constructor.

		@type top: QWidget.
		@param top: The top-level widget. It is used for the
		connection of signals and slots.

		@type parent: QWidget.
		@param parent: The parent widget.
		"""
		QWidget.__init__(self, parent)
		self.top = top
		self.grid = QGridLayout(self)
		width = self.width() / 4
		for i in range(4):
			self.grid.setColumnMinimumWidth(i, width)
		self.pushPreprocess = QPushButton("&Preprocessing", self)
		self.grid.addWidget(self.pushPreprocess, 0, 0)
		self.pushSimulation = QPushButton("Simula&tion", self)
		self.grid.addWidget(self.pushSimulation, 0, 1)
		self.pushClear = QPushButton("Cl&ear", self)
		self.grid.addWidget(self.pushClear, 0, 2)
		self.pushQuit = QPushButton("&Quit", self)
		self.grid.addWidget(self.pushQuit, 0, 3)

		# Connection of signals and slots.
		self.connect(self.pushPreprocess, SIGNAL("clicked()"),
			     self.top.preprocess)
		self.connect(self.pushSimulation, SIGNAL("clicked()"),
			     self.top.simulation)
		self.connect(self.pushClear, SIGNAL("clicked()"),
			     self.top.clear)
		self.connect(self.pushQuit, SIGNAL("clicked()"),
			     self.top.close)


		
