#!/usr/bin/env python

# Copyright (C) 2007, ENPC - INRIA - EDF R&D
#   Author(s): Meryem Ahmed de Biasi
#
# This file is part of the air quality modeling system Polyphemus.
#
# Polyphemus is developed in the INRIA - ENPC joint project-team CLIME and in
# the ENPC - EDF R&D joint laboratory CEREA.
#
# Polyphemus is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# Polyphemus is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# For more information, visit the Polyphemus web site:
#      http://cerea.enpc.fr/polyphemus/

# This module defines dates to be read by Polyphemus.


def number_days(year, month):
	"""
	Computes the number of days in a month of a year.

	@type year: Integer.
	@param year: Year considered.

	@type month: Integer.
	@param month: Month considered.

	@rtype: Integer.
	@return: The number of days in the month and year considered.
	"""
	if month in [1, 3, 5, 7, 8, 10, 12]:
		day = 31
	elif month in [4, 6, 9]:
		day =  30
	else:
		if is_leap_year(year):
			day = 29
		else:
			day = 28
	return day

def is_leap_year(year):
	"""
	Checks whether a year is a leap year.
	"""
	return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)
			
	

class Date:
	"""
	Class to manage dates.
	"""
	def __init__(self, date_str):
		"""
		Constructor.

		@type date_str: String.
		@param date_str: Date in format YYYY-MM-DD_HH-II-SS. The
		hour (HH), minute (II) and second(SS) are optional. Their
		default value is 0. 
		"""
		tmp = "".join(date_str.split("-"))
		tmp = "".join(tmp.split("_"))
                tmp = "".join(tmp.split(" "))
                try:
                    self.year = int(tmp[0:4])
                    self.month = int(tmp[4:6])
                    self.day = int(tmp[6:8])
                    if len(tmp) >= 10:
                        self.hour = int(tmp[8:10])
                    else:
                        self.hour = 0
                    if len(tmp) >= 12:
                        self.minute = int(tmp[10:12])
                    else:
                        self.minute = 0
                    if len(tmp) == 14:
                        self.second = int(tmp[12:14])
                    elif len(tmp)< 14:
                        self.second = 0
                    elif len(tmp) > 14:
                        raise ValueError
                except ValueError:
                    print "The date provided is not valid."
                    

	def return_string(self):
		"""
		Returns the date as a string YYYY-MM-DD_HH-II-SS.
		"""
		s = str(self.year) +"-"
		if self.month < 10:
			s += "0" + str(self.month) + "-"
		else:
			s += str(self.month) + "-"	
		if self.day < 10:
			s += "0" + str(self.day) + "_"
		else:
			s += str(self.day) + "_"
		if self.hour < 10:
			s += "0" + str(self.hour) + "-"
		else:
			s += str(self.hour) + "-"
		if self.minute < 10:
			s += "0" + str(self.minute) + "-"
		else:
			s += str(self.minute) + "-"
		if self.second < 10:
			s += "0" + str(self.second)
		else:
			s += str(self.second)
		return s

	def formate(self):
		"""
		Formates a data as YYYY-MM-DD HH:II:SS.

		@rtype: String.
		@return: The data as a formatted string.
		"""		
		date_form = str(self.year) + "-"
		if self.month < 10:
			date_form += "0" + str(self.month) + "-"
		else:
			date_form += str(self.month) + "-"	
		if self.day < 10:
			date_form += "0" + str(self.day) + " "
		else:
			date_form += str(self.day) + " "
		if self.hour < 10:
			date_form += "0" + str(self.hour) + ":"
		else:
			date_form += str(self.hour) + ":"
		if self.minute < 10:
			date_form += "0" + str(self.minute) + ":"
		else:
			date_form += str(self.minute) + ":"
		if self.second < 10:
			date_form += "0" + str(self.second)
		else:
			date_form += str(self.second)
		return date_form

	def adjust(self):
		"""
		Adjusts the date to make it valid.
		"""
		# Adjust minutes.
		if self.second > 59:
			self.minute += self.second / 60
			self.second = self.second % 60
		elif self.second < 0:
			self.minute += self.second / 60 - 1
			self.second = self.second % 60
		# Adjusts hours.
		if self.minute > 59:
			self.hour += self.minute / 60
			self.minute = self.minute % 60
		elif self.minute < 0:
			self.hour += (self.minute + 1) / 60 - 1
			self.minute = 59 + (self.minute + 1) % 60
		# Adjusts day.
		if self.hour > 23:
			self.day += self.hour / 24
			self.hour = self.hour % 24
		elif self.hour < 0:
			self.day += self.hour / 24 - 1;
			self.hour = 24 + self.hour % 24;
		#Adjusts month.
		if self.day > number_days(self.year, self.month):
			while self.days > number_days(self.year, self.month):
				self.day -= number_days(self.year, self.month)
				self.month += 1
		elif self.day < 1:
			while self.day < 1:
				self.day += number_days(self.year, self.month)
				self.month -= 1
		# Adjusts year.			
		if self.month > 12:
			self.yeaf += self.month / 12
			self.month = self.month % 12
		elif self.month < 1:
			self.year += self.month / 12 - 1
			self.month = 12 + self.month % 12
		
	def get_seconds(self):
		"""
		Computes the number of seconds from the beginning of the year.

		@rtype: Integer.
		@return: The number of seconds since the beginning of year.	
		"""
		# Number of days from the beginning of the year to the
		# beginning of the month.
		days = 0
		for m in range(1, self.month):
			days += number_days(self.year, m)
		days += self.day
		seconds = self.second \
			  + 60 * (self.minute + 60 * (self.hour + 24 * days))
		return seconds

	def add_hours(self, hours):
		"""
		Add hours to a date. The number of hours can be negative.
		
		@type hours: Integer.
		@param hours: The number of hours to add to the date.
		"""
		self.hour += hours
		self.adjust()
		
	def compute_Nt(self, date2, delta):
		"""
		Compute the number of iterations between the date and Date2
		(excluded) with a time step of delta seconds.
		
		@type date2: Date.
		@param date2: Second date considered.
		
		@type delta: Integer.
		@param delta: Time step in seconds.

		@rtype: Integer.
		@return: The number of iterations to perform.
		"""
		sec1 = self.get_seconds()
		sec2 = date2.get_seconds()
		if self.year == date2.year:
			seconds = sec2 - sec1
		else:
			days = 0
			for y in range(self.year, Date2.year):
				if is_leap_year(y):
					days += 366
				else:
					days += 365
			seconds = days * 24 * 3600 + sec2 - sec1
		Nt = seconds / delta
		return Nt

