#!/usr/bin/env python

# Copyright (C) 2007 - 2008, ENPC - INRIA - EDF R&D
#   Author(s): Meryem Ahmed de Biasi
#
# This file is part of the air quality modeling system Polyphemus.
#
# Polyphemus is developed in the INRIA - ENPC joint project-team CLIME and in
# the ENPC - EDF R&D joint laboratory CEREA.
#
# Polyphemus is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 2 of the License, or (at your option)
# any later version.
#
# Polyphemus is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
# more details.
#
# For more information, visit the Polyphemus web site:
#      http://cerea.enpc.fr/polyphemus/

# This module defines a graphical user interface for Polyphemus.

import os, glob

from simulation import *
from classes import *

		
class MainWindow(QMainWindow):
	"""
	Main window of the application.
	"""	
	def __init__(self):
		"""
		Constructor.
		"""
		# Creation of the main window.
		QMainWindow.__init__(self)
		self.main = QWidget(self)
		self.setCentralWidget(self.main)
		self.setWindowTitle("polyphemus_display.py")
		self.setWindowIcon(QIcon(QPixmap("images/logo.png")))

		self.config = PolyphemusConfig()

		self.create_actions()
		self.add_elements()
		self.create_menu()		
		self.resize_grid()
		
		sectionList = [self.selectPath, self.domain, self.dataPath,
			       self.date, self.species]
		for section in sectionList:
			for line in section.lineList:
				self.connect(self,
					     SIGNAL("do_print_friend()"),
					     line.print_friend)
		
	
		self.statusBar().showMessage("Launch the help file with Ctrl+H.")
		

	def create_menu(self):
		"""
		Creates the menus to put in the menu bar.
		"""
		self.file = QMenu("File", self)
		self.menuBar().addMenu(self.file)
		self.file.addAction(self.save_action)
		self.file.addAction(self.load_action)
		self.file.addAction(self.clear_action)
		self.file.addAction(self.close_action)

		self.edit = QMenu("Edit", self)
		self.menuBar().addMenu(self.edit)
		self.edit.addAction(self.source_action)
		self.edit.addAction(self.levels_action)

		self.computation = QMenu("Computation", self)
		self.menuBar().addMenu(self.computation)
		self.computation.addAction(self.preprocess_action)
		self.computation.addAction(self.simulation_action)

		self.image_action = QAction("Save i&mage", self)
		self.image_action.setShortcut(Qt.CTRL + Qt.Key_M)
		self.connect(self.image_action, SIGNAL("triggered()"),
			     self.image.save)
		self.image_menu =  QMenu("Image", self)
		self.menuBar().addMenu(self.image_menu)
		self.image_menu.addAction(self.image_action)
		self.image_menu.addAction(self.image.for_action)
		self.image_menu.addAction(self.image.back_action)
		
		self.help =  QMenu("Help", self)
		self.menuBar().addMenu(self.help)
		self.help.addAction(self.about_action)
		self.help.addAction(self.help_action)
		
		
	def create_actions(self):
		"""
		Creates the actions.
		"""
		self.save_action = QAction("&Save configuration file", self)
		self.save_action.setShortcut(Qt.CTRL + Qt.Key_S)
		self.connect(self.save_action, SIGNAL("triggered()"),
			     self.save_config)
		self.load_action = QAction("L&oad configuration file", self)
		self.load_action.setShortcut(Qt.CTRL + Qt.Key_O)
		self.connect(self.load_action, SIGNAL("triggered()"),
			     self.load_config)
		self.clear_action = QAction("Cl&ear", self)
		self.clear_action.setShortcut(Qt.CTRL + Qt.Key_E)
		self.connect(self.clear_action, SIGNAL("triggered()"),
			     self.clear)
		self.close_action = QAction("&Quit", self)
		self.close_action.setShortcut(Qt.CTRL + Qt.Key_Q)
		self.connect(self.close_action, SIGNAL("triggered()"),
			     self.close)
		self.source_action = QAction("Edit sou&rce file", self)
		self.source_action.setShortcut(Qt.CTRL + Qt.Key_R)
		self.connect(self.source_action, SIGNAL("triggered()"),
			     self.edit_source)
		self.levels_action = QAction("Edit &level file", self)
		self.levels_action.setShortcut(Qt.CTRL + Qt.Key_L)
		self.connect(self.levels_action, SIGNAL("triggered()"),
			     self.edit_levels)

		self.preprocess_action = QAction("&Preprocessing", self)
		self.preprocess_action.setShortcut(Qt.CTRL + Qt.Key_P)
		self.connect(self.preprocess_action, SIGNAL("triggered()"),
			     self.preprocess)
		
		self.simulation_action = QAction("Simula&tion", self)
		self.simulation_action.setShortcut(Qt.CTRL + Qt.Key_T)
		self.connect(self.simulation_action, SIGNAL("triggered()"),
			     self.simulation)

		self.about_action = QAction("&About", self)
		self.about_action.setShortcut(Qt.CTRL + Qt.Key_A)
		self.connect(self.about_action, SIGNAL("triggered()"),
			     self.about)
		self.help_action = QAction("&Help", self)
		self.help_action.setShortcut(Qt.CTRL + Qt.Key_H)
		self.connect(self.help_action, SIGNAL("triggered()"),
			     self.help_box)

		
	def add_elements(self):
		"""
		Adds the various elements of the window to the grid.
		"""
		# Grid with 3 columns and 7 rows.
		self.grid = QGridLayout(self.main)
		
		self.selectPath = SelectPath(self.main, self.config)
		self.grid.addWidget(self.selectPath, 0, 1)

		config_image = "work/results/disp.cfg"
		self.image = ImageViewer(self.main, self, 5, 5,
					 100, config_image)
		self.grid.addWidget(self.image, 0, 0, 7, 1)
		
		self.domain = SectionDomain(self.main, self.config)
		self.grid.addWidget(self.domain, 1, 1)

		self.dataPath = DataPath(self.main, self.config)
		self.grid.addWidget(self.dataPath, 2, 1)

		self.date = SectionDate(self.main, self.config)
		self.grid.addWidget(self.date, 3, 1)

		self.species = SectionSpecies(self.main, self.config)
		self.grid.addWidget(self.species, 4, 1)

		self.kz = SectionKz(self.main, self.config)
		self.grid.addWidget(self.kz, 5, 1)
     
		self.button = GroupButton(self, self.main)
		self.grid.addWidget(self.button, 6, 1)

	def resize_grid(self):
		"""
		Resizes the widget and the columns and rows of its layout.
		"""
		self.resize(QSize(1020, 440))

		# Defines the minimum width of the columns.
		self.grid.setColumnMinimumWidth(0, 400)
		self.grid.setColumnMinimumWidth(1, 500)
		
		# Defines the minimum height of the rows.
		self.grid.setRowMinimumHeight(0, 40)
		self.grid.setRowMinimumHeight(1, 120)
		self.grid.setRowMinimumHeight(2, 80)
		self.grid.setRowMinimumHeight(3, 40)
		self.grid.setRowMinimumHeight(4, 80)
		self.grid.setRowMinimumHeight(5, 40)
		self.grid.setRowMinimumHeight(6, 40)	
		
	def save_config(self):
		"""
		Saves all parameters of the simulation in a configuration
		file. If the file is not found, nothing is done.
		"""
		import talos
		try:
			f = str(QFileDialog(self.file).getSaveFileName(self,
				"Select the file in which to save.",
			        "./configuration.cfg", ""))
			if self.kz.comboBoxKz.currentIndex() == 1:
				self.config.parameter.value = "TM"
			else:
				self.config.parameter.value = "Louis"	
			
			config = []
			for item in self.config.list:
				config.append([item.name, item.section,
					       item.value])
			talos.write_configuration_file(config, f)
			
		except:
			pass

	def load_config(self):
		"""
		Loads all parameters of the simulation from a configuration
		file. If the file is not found, nothing is done.
		"""
		try:
			f = str(QFileDialog(self.file).getOpenFileName(self,
				"Select the configuration file to open.",
			        "./configuration.cfg", ""))
			self.config.load(f)
			self.kz.print_friend()
			self.emit(SIGNAL("do_print_friend()"), ())
		except:
			pass

	def edit_source(self):
		"""
		Opens the source file in emacs, allows the user to modify it
		then reloads it if necessary. 
		"""
		import commands
		f = str(QFileDialog(self.edit).getOpenFileName(self,
			"Select the source file to edit.",
			self.config.source.value,
			"Data file (*.dat);; All files (*)"))
		
		if f != "":
			commands.getstatusoutput("emacs " + f)
			friend = self.species.selectSource.line
			dial = DialogFile(self, self.main, friend, f,
			"Source file has been modified. Do you wish to reload it?")
			dial.show()

	def edit_levels(self):
		"""
		Opens the levels file in emacs, allows the user to modify it
		then reloads it if necessary. 
		"""
		import commands
		f = str(QFileDialog(self.edit).getOpenFileName(self,
			"Select the levels file to edit.",
			self.config.levels.value,
			"Data file (*.dat);; All files (*)"))		
		if f !="":
			commands.getstatusoutput("emacs " + f)
			friend = self.domain.selectLevels.line
			dial = DialogFile(self, self.main, friend, f,
			"Levels file has been modified. Do you wish to reload it?")
			dial.show()

	def help_box(self):
		"""
		Opens a dialog to display help information.
		"""
		help = HelpBox(self.main, "help/help.html")
		help.setWindowTitle("polyphemus_display.py Help File.")
		help.resize(QSize(900, 800))
		
	def about(self):
		"""
		Opens a dialog to display copyright information.
		"""
		about = HelpBox(self.main, "help/about.html")
		about.setWindowTitle("About polyphemus_display.py.")
		about.resize(QSize(650, 360))

	def preprocess(self):
		"""
		Performs preprocessing.
		"""
		import commands
		
		self.statusBar().showMessage("Preprocessing")
		
		poly_path = self.config.path.value
		
		date1_str = self.config.date1.value
		date1 = Date(date1_str)
 		date2_str = self.config.date2.value
		date2 = Date(date2_str)

		# The end date is excluded so we add two hours to be able to
		# perform simulation until the end date.
		date2.add_hours(2)
		date2.adjust()
		date2_str = date2.return_string()
		
		# Creation of the simulation.
		simulation = Simulation(self.config.species.value)
            
		# Working directories.
		ref_config_dir = "configuration/"
		
		working_dir = "work/"
		
		config_dir = working_dir + "configuration/"
		data_dir = working_dir + "preprocessing/"
		
		# Delete and re-create some files.
		list_file = glob.glob(config_dir + "*.cfg")
		list_file += glob.glob(data_dir + "ground/*.bin")
		list_file += glob.glob(data_dir + "meteo/*.bin")

		if list_file is not []:
			for filename in list_file:
				os.remove(filename)
			

		meteo_file = self.config.meteo.value
		ground_dir = self.config.ground.value

		# Dictionary used to replace values in all configuration files.
		preprocess_dict = {"%Date1%": date1_str, "%Date2%": date2_str, 
				   "%x_min%": self.config.x_min.value,
				   "%Delta_x%": self.config.Delta_x.value,
				   "%Nx%": self.config.Nx.value,
				   "%y_min%": self.config.y_min.value,
				   "%Delta_y%": self.config.Delta_y.value,  
				   "%Ny%": self.config.Ny.value,
				   "%Nz%": self.config.Nz.value,
				   "%levels%" : self.config.levels.value,
				   "%species%": self.config.species.value,
				   "%quantity_ic%": self.config.initial.value, 
				   "%source%": self.config.source.value,
				   "%param%": self.config.parameter.value,
				   "%meteo_file%": meteo_file,
				   "%ground_dir%": ground_dir,
				   "%config_dir%": config_dir[:-1], 
				   "%data_dir%": data_dir[:-1],
				   "%polyphemus_path%": poly_path[:-1]}

		config = Configuration([ref_config_dir + "species.dat",
					ref_config_dir + "levels.dat",
					ref_config_dir + "land_data_usgs.dat",
					ref_config_dir + "roughness-usgs.dat"])
		config.SetConfiguration(preprocess_dict, "path", config_dir)
		config.ProceedReplacement()	
        
		# luc-usgs.
		program = Program(poly_path + 
				  "preprocessing/ground/luc-usgs",
				  format = " %a ")
		program.SetConfiguration([ref_config_dir + "general.cfg",
					  ref_config_dir + "luc-usgs.cfg"])
		program.SetConfiguration(preprocess_dict, "path", config_dir)
		simulation.AddProgram(program)

            
		# roughness.
		program = Program(poly_path + 
				  "preprocessing/ground/roughness",
				  format = " %a ")
		program.SetConfiguration([ref_config_dir +"general.cfg",
					  ref_config_dir + "roughness.cfg"])
		program.SetConfiguration(preprocess_dict, "path", config_dir)
		simulation.AddProgram(program)
	    
		# MM5-meteo.
		program = Program(poly_path +
				  "preprocessing/meteo/MM5-meteo",
				  format = " %a " + date1_str
				  + " " + date2_str)
		program.SetConfiguration([ref_config_dir + "general.cfg",
					  ref_config_dir + "MM5-meteo.cfg"])
		program.SetConfiguration(preprocess_dict, "path", config_dir)
		simulation.AddProgram(program)
        
		# Kz_TM if necessary.
		if self.config.parameter.value == "TM":
			program = Program(poly_path +
					  "preprocessing/meteo/Kz_TM",
					  format = " %a " + date1_str
					  + " " + date2_str)
			program.SetConfiguration([ref_config_dir +
						  "general.cfg",
						  ref_config_dir +
						  "MM5-meteo.cfg"]) 
			program.SetConfiguration(preprocess_dict,
						 "path", config_dir)
			simulation.AddProgram(program)

		# Launch of the simulation.
		simulation.Run()

			

	def simulation(self):
		"""
		Performs a simulation using results of self.preprocessing.
		"""
		import commands

		self.statusBar().showMessage("Simulation")

		# Cleans the image.
		self.image.clear()

		poly_path = self.config.path.value

		date1_str = self.config.date1.value
		date1 = Date(date1_str)
 		date2_str = self.config.date2.value
		date2 = Date(date2_str)

		Nt = date1.compute_Nt(date2, 600)
		Nt_save = Nt / 6
            
		# Creation of the simulation.
		simulation = Simulation(self.config.species.value)
            
		# Working directories.
		ref_config_dir = "configuration/"

		working_dir = "work/"
	    
		config_dir = working_dir + "configuration/"
		data_dir = working_dir + "preprocessing/"
		result_dir = working_dir + "results/"

		# Delete and re-create some files.
		list_file = glob.glob(result_dir + "*.cfg")
		list_file += glob.glob(result_dir + "*.bin")

		if list_file is not []:
			for filename in list_file:
				os.remove(filename)

		
		# Levels for which concentrations are saved: all levels used.
		Nzr = int(self.config.Nz.value)
		zrange = range(Nzr)
		levels_save = ""
		for k in zrange:
			levels_save += str(k) + " "

		simulation_dict = {"%Date1%": date1_str, "%Date2%": date2_str,
				   "%x_min%": self.config.x_min.value,
				   "%Delta_x%": self.config.Delta_x.value,
				   "%Nx%": self.config.Nx.value,
				   "%y_min%": self.config.y_min.value,
				   "%Delta_y%": self.config.Delta_y.value,  
				   "%Ny%": self.config.Ny.value,
				   "%Nz%": self.config.Nz.value,
				   "%Nt_simu%": Nt,
				   "%Nt_save%": Nt_save,
				   "%levels%" : self.config.levels.value,
				   "%levels_save%": levels_save,
				   "%species%": self.config.species.value,
				   "%quantity_ic%": self.config.initial.value,
				   "%param%": self.config.parameter.value,
				   "%source%": self.config.source.value,
				   "%config_dir%": config_dir[:-1], 
				   "%result%": result_dir[:-1],
				   "%data_dir%": data_dir[:-1],
				   "%polyphemus_path%": poly_path[:-1]}

	
		# polair3D.
		program = Program(poly_path + "driver/polair3d",
				  format = " %a")
		program.SetConfiguration(ref_config_dir + "polair3d.cfg")
		program.SetConfiguration(simulation_dict, "path", config_dir)
		
		# Additional configuration files.
		config = Configuration([ref_config_dir + "species.dat",
					ref_config_dir + "polair3d-data.cfg",
					ref_config_dir + "polair3d-saver.cfg"])
		config.SetConfiguration(simulation_dict, "path", config_dir)
		config.ProceedReplacement()
		
		simulation.AddProgram(program)

		# Launch of the simulation.
		simulation.Run()

		# Display of results.
		self.statusBar().showMessage("Displaying results")
		config_disp = Configuration(ref_config_dir + "disp.cfg")
		config_disp.SetConfiguration(simulation_dict, "path", result_dir)
		config_disp.ProceedReplacement()

		config_disp = result_dir + "disp.cfg" 
	
		self.image.polyphemusDisplay.config = config_disp
		self.image.polyphemusDisplay.map = None
		self.image.polyphemusDisplay.data = None
		self.image.polyphemusDisplay.level = 0
		self.image.polyphemusDisplay.step = 0
		self.image.print_friend()
		self.image.polyphemusDisplay.dispcf()


	def clear(self):
		"""
		Clears the window.
		"""
		self.statusBar().showMessage("Dialog cleared.")
		self.selectPath.clear()
		self.image.clear()
		self.domain.clear()
		self.date.clear()
		self.species.clear()
		self.kz.clear()
		self.dataPath.clear()


if __name__ == "__main__":
	app = QApplication(sys.argv)
	f = MainWindow()
	f.show()
	sys.exit(app.exec_())
